﻿using System;
using System.Threading;
using System.Security.AccessControl;
using System.Security.Principal;
using System.Text.RegularExpressions;

namespace Z
{
    public class ZMutex : IDisposable
    {

        public bool hasHandle = false;
        Mutex mutex;

        private void InitMutex(string MutexID)
        {
            string appGuid = MutexID;
            string mutexId = string.Format("Global\\{{{0}}}", appGuid);
            mutex = new Mutex(false, mutexId);

            var allowEveryoneRule = new MutexAccessRule(new SecurityIdentifier(WellKnownSidType.WorldSid, null), MutexRights.FullControl, AccessControlType.Allow);
            var securitySettings = new MutexSecurity();
            securitySettings.AddAccessRule(allowEveryoneRule);
            mutex.SetAccessControl(securitySettings);
        }

        public ZMutex(string GlobalMutexID, int timeOut)
        {
            if (GlobalMutexID == "") throw new Exception("GlobalMutexID should be non-empty");
            if (!Regex.IsMatch(GlobalMutexID, "^[a-zA-Z0-9_.-]*$")) throw new Exception("GlobalMutexID should contain only ascii alphabetic characters, numbers, dots \"-\" or \"_\"");

            InitMutex(GlobalMutexID);
            try
            {
                if (timeOut < 0)
                    hasHandle = mutex.WaitOne(Timeout.Infinite, false);
                else
                    hasHandle = mutex.WaitOne(timeOut, false);

                if (hasHandle == false)
                    throw new TimeoutException("Timeout waiting for exclusive access on " + GlobalMutexID);
            }
            catch (AbandonedMutexException)
            {
                hasHandle = true;
            }
        }

        public void Dispose()
        {
            if (mutex != null)
            {
                if (hasHandle)
                    mutex.ReleaseMutex();
                mutex.Dispose();
            }
        }

    }
}
